@@ -11,13 +11,12 @@ use crate::parse::{HostLang, TypeDeclarations};
 use proc_macro2::Ident;
 use std::ops::Deref;
 use syn::parse::{Parse, ParseStream};
-use syn::{ForeignItemType, LitStr, PatType, Path, Token, Type};
+use syn::{ForeignItemType, LitStr, Path, PatType, Token, Type};
 
 use crate::parsed_extern_fn::ParsedExternFn;
 
 mod errors;
 mod parse;
-mod to_tokens;
 
 mod built_in_types;
 mod parsed_extern_fn;
