@@ -1,11 +1,12 @@
 use proc_macro2::Ident;
 use syn::parse::{Parse, ParseStream};
-use syn::Token;
+use syn::{LitStr, Token};
 
 #[derive(Default)]
 pub(super) struct FunctionAttributes {
     pub associated_to: Option<Ident>,
     pub is_initializer: bool,
+    pub swift_name: Option<LitStr>,
 }
 
 impl FunctionAttributes {
@@ -15,12 +16,16 @@ impl FunctionAttributes {
                 self.associated_to = Some(ident);
             }
             FunctionAttr::Init => self.is_initializer = true,
+            FunctionAttr::SwiftName(name) => {
+                self.swift_name = Some(name);
+            }
         }
     }
 }
 
 pub(super) enum FunctionAttr {
     AssociatedTo(Ident),
+    SwiftName(LitStr),
     Init,
 }
 
@@ -35,6 +40,12 @@ impl Parse for FunctionAttr {
 
                 FunctionAttr::AssociatedTo(value)
             }
+            "swift_name" => {
+                input.parse::<Token![=]>()?;
+                let value: LitStr = input.parse()?;
+
+                FunctionAttr::SwiftName(value)
+            }
             "init" => FunctionAttr::Init,
             _ => panic!("TODO: Return spanned error"),
         };
